/*
  LiquidCrystal_MCP23x17 library - Hello World sample
  see http://freeduino.ru for details
  
  sample is based on original Arduino LiquidCrystal sample
  with only difference, that we use MCP23x17-based shield
  for interfacing with LCD display
*/

//For SPI version of shield we include SPI.h:
#include <SPI.h>
////For I2C version of shield we include Wire.h:
//#include <Wire.h>

//include the library code
#include <MCP23xxx.h>
#include <LiquidCrystal_MCP23x17.h>

//For SPI version of shield we create CMCP23S17 object:
CMCP23S17 MCP;
////For I2C version of shield we create CMCP23017 object:
//CMCP23017 MCP;

//LCD object
LiquidCrystal_MCP23x17 lcd;

void setup() {
  //For SPI version of shield we configure SPI bus, and MCP object with CS line (10) and address (0)
  SPI.begin();
  MCP.init(10, 0);

//  //For I2C version of shield we configure I2C bus, and MCP object with address (0)
//  Wire.begin();      //TWBR = 12;  // <- uncomment this for 400kHz I2C
//  MCP.init(0);
  
  //here we configure LCD object to work through specified MCP object
  lcd.init(MCP);
  //...and turn on backlight
  lcd.Backlight(1);

  //...all the following code will be the same for original LiquidCrystal library and MCP23x17-based
  
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print("hello, world!");
  // Enter key is on 12 pin of MCP port expander
  MCP.pinMode(12, INPUT);
}

void loop() {
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0, 1);
  // print the number of seconds since reset:
  lcd.print(millis()/1000);
  // if Enter pressed, print some text
  if (MCP.digitalRead(12)==0)
    lcd.print(" Enter pressed");
  // or else, print spaces to wipe out old text
  else
    lcd.print("              ");
}