#ifndef MCP23XXX_H_INCLUDED
#define MCP23XXX_H_INCLUDED

#include <inttypes.h>
#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

class CIOExpander
{
  public:
  virtual void pinMode(uint8_t pin, uint8_t mode);
  virtual void digitalWrite(uint8_t pin, uint8_t val);
  virtual int digitalRead(uint8_t pin);

  virtual void pinMode16(uint16_t mode);
  virtual void digitalWrite16(uint16_t val);
  virtual uint16_t digitalRead16();
  virtual void pinMode8(uint8_t mode);
  virtual void digitalWrite8(uint8_t val);
  virtual uint8_t digitalRead8();
};

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> class CMCP23xxx : public CIOExpander
{
  public:
  virtual void write_reg16(uint8_t addr, uint16_t data);
  virtual uint16_t read_reg16(uint8_t addr);
  virtual void write_reg8(uint8_t addr, uint8_t data);
  virtual uint8_t read_reg8(uint8_t addr);

  void pinMode(uint8_t pin, uint8_t mode);
  void digitalWrite(uint8_t pin, uint8_t val);
  int digitalRead(uint8_t pin);

  void pinMode16(uint16_t mode);
  void digitalWrite16(uint16_t val);
  uint16_t digitalRead16();
  void pinMode8(uint8_t mode);
  void digitalWrite8(uint8_t val);
  uint8_t digitalRead8();

  uint8_t m_rw_cmd;

  const static uint8_t IODIR = addrIODIR;
  const static uint8_t GPIO  = addrGPIO;
  const static uint8_t IOCON = addrIOCON;

  inline uint8_t  low_byte(uint16_t x) { return x & 0x00FF; }
  inline uint8_t  high_byte(uint16_t x) { return x >> 8; }
  inline uint16_t make_uint16(uint8_t lbyte, uint8_t hbyte) { return (uint16_t)hbyte<<8 | (uint16_t)lbyte; }

};

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::pinMode(uint8_t pin, uint8_t mode)
{
  uint8_t tmp_REG = IODIR + ((pin & 8) >> 3);
  pin&= 7;
  if (mode == INPUT)
    write_reg8(tmp_REG, read_reg8(tmp_REG) | 1<<pin);
  else
    write_reg8(tmp_REG, read_reg8(tmp_REG) & ~(1<<pin));
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalWrite(uint8_t pin, uint8_t val)
{
  uint8_t tmp_REG = GPIO + ((pin & 8) >> 3);
  pin&= 7;
  if (val)
    write_reg8(tmp_REG, read_reg8(tmp_REG) | 1<<pin);
  else
    write_reg8(tmp_REG, read_reg8(tmp_REG) & ~(1<<pin));
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> int CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalRead(uint8_t pin)
{
  return (read_reg8(GPIO + ((pin & 8) >> 3)) & 1<<(pin&7)) != 0;
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::pinMode16(uint16_t mode)
{
  write_reg16(IODIR, mode);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalWrite16(uint16_t val)
{
  write_reg16(GPIO, val);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint16_t CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalRead16()
{
  return read_reg16(GPIO);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::pinMode8(uint8_t mode)
{
  write_reg8(IODIR, mode);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalWrite8(uint8_t val)
{
  write_reg16(GPIO, val);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint8_t CMCP23xxx<addrIODIR, addrGPIO, addrIOCON>::digitalRead8()
{
  return read_reg8(GPIO);
}

#ifdef TwoWire_h
template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> class CMCP230xx : public CMCP23xxx <addrIODIR, addrGPIO, addrIOCON>
{
  public:
  CMCP230xx();
  CMCP230xx(uint8_t hw_addr);

  void init(uint8_t hw_addr = 0);
  void write_reg16(uint8_t addr, uint16_t data);
  uint16_t read_reg16(uint8_t addr);
  void write_reg8(uint8_t addr, uint8_t data);
  uint8_t read_reg8(uint8_t addr);
};

#if defined(ARDUINO) && ARDUINO >= 100
#define WIRE_WRITE(x) Wire.write(x)
#define WIRE_READ() Wire.read()
#else
#define WIRE_WRITE(x) Wire.send(x)
#define WIRE_READ() Wire.receive()
#endif

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::CMCP230xx()
{
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::CMCP230xx(uint8_t hw_addr)
{
	init(hw_addr);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::init(uint8_t hw_addr)
{
  this->m_rw_cmd = 0x20 | hw_addr;
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::write_reg16(uint8_t addr, uint16_t data)
{
  Wire.beginTransmission(this->m_rw_cmd);
  WIRE_WRITE(addr);
  WIRE_WRITE(this->low_byte(data));
  WIRE_WRITE(this->high_byte(data));
  Wire.endTransmission();
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint16_t CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::read_reg16(uint8_t addr)
{
  uint8_t lbyte, hbyte;
  Wire.beginTransmission(this->m_rw_cmd);
  WIRE_WRITE(addr);
  Wire.endTransmission();
  Wire.requestFrom(this->m_rw_cmd, (uint8_t)2);
  while (Wire.available() < 2);
  lbyte = WIRE_READ();
  hbyte = WIRE_READ();
  return this->make_uint16(lbyte, hbyte);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::write_reg8(uint8_t addr, uint8_t data)
{
  Wire.beginTransmission(this->m_rw_cmd);
  WIRE_WRITE(addr);
  WIRE_WRITE(data);
  Wire.endTransmission();
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint8_t CMCP230xx<addrIODIR, addrGPIO, addrIOCON>::read_reg8(uint8_t addr)
{
  Wire.beginTransmission(this->m_rw_cmd);
  WIRE_WRITE(addr);
  Wire.endTransmission();
  Wire.requestFrom(this->m_rw_cmd, (uint8_t)1);
  while (Wire.available() < 1);
  return WIRE_READ();
}

typedef CMCP230xx<0x00, 0x09, 0x05> CMCP23008;
typedef CMCP230xx<0x00, 0x12, 0x0A> CMCP23017;
#endif

#ifdef _SPI_H_INCLUDED
template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> class CMCP23Sxx : public CMCP23xxx <addrIODIR, addrGPIO, addrIOCON>
{
  public:
  CMCP23Sxx();
  CMCP23Sxx(uint8_t ss_pin, uint8_t hw_addr = 0, bool reinit_HAEN = true);

  void init(uint8_t ss_pin, uint8_t hw_addr = 0, bool reinit_HAEN = true);
  void write_reg16(uint8_t addr, uint16_t data);
  uint16_t read_reg16(uint8_t addr);
  void write_reg8(uint8_t addr, uint8_t data);
  uint8_t read_reg8(uint8_t addr);

  uint8_t m_ss_pin;

  const static uint8_t HAEN = 0x08;
};

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::CMCP23Sxx()
{
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::CMCP23Sxx(uint8_t ss_pin, uint8_t hw_addr, bool reinit_HAEN)
{
	init(ss_pin, hw_addr, reinit_HAEN);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::init(uint8_t ss_pin, uint8_t hw_addr, bool reinit_HAEN)
{
  this->m_ss_pin = ss_pin;
  ::digitalWrite(ss_pin, HIGH);
  ::pinMode(ss_pin, OUTPUT);

  this->m_rw_cmd = 0x40;
  if (reinit_HAEN) {
    write_reg8(this->IOCON, HAEN);
  }
  this->m_rw_cmd |= hw_addr << 1;
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::write_reg16(uint8_t addr, uint16_t data)
{
  ::digitalWrite(this->m_ss_pin, LOW);
  SPI.transfer(this->m_rw_cmd);
  SPI.transfer(addr);
  SPI.transfer(this->low_byte(data));
  SPI.transfer(this->high_byte(data));
  ::digitalWrite(this->m_ss_pin, HIGH);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint16_t CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::read_reg16(uint8_t addr)
{
  uint8_t lbyte, hbyte;
  ::digitalWrite(this->m_ss_pin, LOW);
  SPI.transfer(this->m_rw_cmd | 1);
  SPI.transfer(addr);
  lbyte = SPI.transfer(0);
  hbyte = SPI.transfer(0);
  ::digitalWrite(this->m_ss_pin, HIGH);
  return this->make_uint16(lbyte, hbyte);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> void CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::write_reg8(uint8_t addr, uint8_t data)
{
  ::digitalWrite(this->m_ss_pin, LOW);
  SPI.transfer(this->m_rw_cmd);
  SPI.transfer(addr);
  SPI.transfer(data);
  ::digitalWrite(this->m_ss_pin, HIGH);
}

template <uint8_t addrIODIR, uint8_t addrGPIO, uint8_t addrIOCON> uint8_t CMCP23Sxx<addrIODIR, addrGPIO, addrIOCON>::read_reg8(uint8_t addr)
{
  uint8_t lbyte;
  ::digitalWrite(this->m_ss_pin, LOW);
  SPI.transfer(this->m_rw_cmd | 1);
  SPI.transfer(addr);
  lbyte = SPI.transfer(0);
  ::digitalWrite(this->m_ss_pin, HIGH);
  return lbyte;
}

typedef CMCP23Sxx<0x00, 0x09, 0x05> CMCP23S08;
typedef CMCP23Sxx<0x00, 0x12, 0x0A> CMCP23S17;
#endif

#endif
