/*
  MCP23xxx library - Repeat sample
  This sample reads pin state of 1'st MCP23x17 chip, and set the same values on 2'nd chip
  Try it with freeduino.ru Expander Shield
  see http://freeduino.ru for details
*/

////For SPI version of shield we include SPI.h:
//#include <SPI.h>
//For I2C version of shield we include Wire.h:
#include <Wire.h>

//include the library code
#include <MCP23xxx.h>

////For SPI version of shield we create two CMCP23S17 objects:
//CMCP23S17 mcp23_1;
//CMCP23S17 mcp23_2;
//For I2C version of shield we create two CMCP23017 object:
CMCP23017 mcp23_1;
CMCP23017 mcp23_2;
//But we cannot initialize it now, because the bus is not ready 

void setup()
{
//  //For SPI version of shield we configure SPI bus, and MCP objects with CS line (10) and addresses (0 and 1)
//  SPI.begin();
//  mcp23_1.init(10, 0);
//  mcp23_2.init(10, 1);
  //For I2C version of shield we configure I2C bus, and MCP object with addresses (0 and 1)
  Wire.begin();      //TWBR = 12;  // <- uncomment this for 400kHz I2C
  mcp23_1.init(0);
  mcp23_2.init(1);

  //We can configure all pins of 1'st chip as inputs,
  //and all pins of 2'nd chip as outputs in a loop   
  for (int i=0; i<16; i++) {
    mcp23_1.pinMode(i, INPUT);
    mcp23_2.pinMode(i, OUTPUT);
  }
  //or, we can configure all 16 pins at once, calling pinMode16 method
  //mcp23_1.pinMode16(0x0ffff);
  //mcp23_2.pinMode16(0x00000);
}

void loop()
{
  //We can read all pins of 1'st chip and write same values to 2'nd in a loop
  for (int i=0; i<16; i++) {
    mcp23_2.digitalWrite(i, mcp23_1.digitalRead(i));
  }
  //or, we can do it at once with digitalRead16 and digitalWrite16 methods 
  //mcp23_2.digitalWrite16(mcp23_1.digitalRead16());
}
